

/*


To add new features you'll probably need to modify the library itself. I tried to expose as
much functionality as possible so you can add or change features by overriding procs.

Every mob has a proc called "movement". This proc is automatically called each tick and
handles the mob's movement. The default behavior is to check for keyboard input and call
procs to perform various actions.


  -- Version History --

Version 1.1
* Added world.dm and changed how the global movement loop works.
* Added collision check for hitting a ceiling.
* Mobs can now bump turfs and dense objs by default.
* Added support in the default behavior for movement to support direct
  changes to a mob's loc. You don't need to call set_pos to change a mob's
  location, you can set its loc directly.
* Further split the movement behavior by adding the slow_down and gravity procs
* Added the on_left, on_right, on_top, and on_bottom variables.
* Made pixel_move return 1 if any movement was made and 0 otherwise.
* Removed the set_action and compute_state procs. A mob's icon_state is now
  determined solely by set_state which is called from the movement proc by default.

  -- Files --

Library
	keyboard.dm
	mob-movement.dm
	pixel-movement.dm

demo
	demo.dm
	keyboard.dmf
	icons.dmi
	map.dmm

atom
	vars
		px, py, pz
			The atom's position in pixels.

		pwidth, pheight, pdepth
			The size of the atom's bounding box. The box is placed up and right from the atom's
			px/py position.

		offset_x, offset_y
			The pixel offset applied to the atom. This is used when the bottom left of the atom's
			icon is not the bottom left of its bounding box.

	procs
		stepped_on
			This is called when an object is standing on the atom. src is the atom being stepped on.
			This is defined for all atoms but generally used for turfs.

mob
	vars
		vel_x, vel_y, vel_z
			The mob's velocity in the x, y, and z directions.

		on_left
			The number of dense objects to the left of the mob.

		on_right
			The number of dense objects to the right of the mob.

		on_top
			The number of dense objects above the mob (in the y direction).

		on_bottom
			The number of dense objects below the mob (in the y direction).

		on_ground
			The number of dense objects below the mob (in the z direction).

	procs
		movement()
			Automatically called every tick by a global loop. Handles the object's
			movement. There is no default behavior for atom/movable.

		can_bump(atom/a)
			Returns 1 if src can bump the atom, otherwise returns ..().
			This allows for more complex density than just "all dense objects bump all
			other dense objects". For example, players cannot move through fences but
			bullets can, so player.can_bump(fence) = 1 but bullet.can_bump(bullet) = 0.

		pixel_move(dpx, dpy, dpz = 0)
			Figures out if the desired move is valid. It resolves invalid moves and then calls set_pos.

		set_pos(nx, ny, nz, map_z = z)
			Sets the object's px, py, and pz to nx, ny, and nz. Updates the object's loc and
			calls set_camera (if the object is a mob). If you want to move an object to
			a position this is the function to call.

			In other BYOND games you'd write this:

				mob.loc = locate(5,6,1)

			With this library you'd write:

				mob.set_pos(5 * 32, 6 * 32, 0, 1)

mob
	vars
		on_ground
			1 if the mob is standing on a dense object, 0 otherwise.

		base_state
			The prefix of all mob icon states. If a base state is specified, icon states will
			have the form [base state]-[action_state]-[dir_state], otherwise they will just
			be [action_state]-[dir_state].

		jumped
			Pressing a key does not cause the player to jump. Jumping is handled by the movement
			proc. The jumped var is a flag that tells the movement proc when the player jumped.
			When jumped = 1, the movement proc will process the jump and clear the flag.

	procs
		movement
			The default action for a mob's movement proc is to process keyboard input and
			determine what action should be performed. It calls move, climb, and jump to
			handle each action. You can override those procs to change the behavior without
			having to mess with the movement proc.

		move
			Called to handle the action of walking left, right, up, or down.

		jump
			Called when the mob jumps.

		can_jump
			Called to check if the mob is allowed to jump. The default behavior requires that you
			be standing on the ground and not be holding onto a ladder.

		set_camera
			Sets the client's pixel offset.

*/