#define DEBUG

world
	tick_lag = 0.25
	map_format = ISOMETRIC_MAP

atom
	// we need these vars for turfs and mobs at least. We probably don't
	// need them for areas, but it's easy to define them for all atoms.
	var
		// px,py,pz is your position on the map in pixels
		px = 0
		py = 0
		pz = 0

		// pwidth/pheight/pdepth determine the dimensions of your bounding box
		pwidth = 32
		pheight = 32
		pdepth = 0

	New()
		..()
		px = 32 * x
		py = 32 * y
		pz = pixel_z

	proc
		// we don't want to take an atom as an argument because we'll be using
		// this proc to check if a move would put an object inside another, we
		// want to check if the move is ok without actually performing it.
		// So, we take the query point, which is defined by the point qx,qy and
		// the size of the atom's bounding box: qw x qh.
		inside(qx,qy,qw,qh,qz,qd)
			if(qx <= px - qw) return 0
			if(qx >= px + pwidth) return 0
			if(qy <= py - qh) return 0
			if(qy >= py + pheight) return 0
			if(qz >= pz + pdepth) return 0
			if(qz <= pz - qd) return 0
			return 1

		stepped_on(mob/m)

mob
	animate_movement = 0

	var
		offset_x = 0
		offset_y = 0

	proc
		x_bump(atom/a)
		y_bump(atom/a)
		z_bump(atom/a)

		can_bump(atom/a)
			// You can bump all turfs, even non-dense ones, otherwise
			// you'd fall right through the ground.
			if(isturf(a))
				return 1

			if(isobj(a))
				return a.density

			return 0

		// The pixel_move proc moves a mob by (dpx, dpy, dpz) pixels.
		pixel_move(dpx, dpy, dpz = 0)

			// We could intelligently look for all nearby tiles that you might hit
			// based on your position and direction of the movement, but instead
			// we'll just check every nearby tile.
			for(var/atom/t in oview(2,src))
				if(!can_bump(t)) continue

				// You cannot be inside t already, so if the move doesn't put you
				// inside t then we can ignore that turf.
				if(!t.inside(px+dpx,py+dpy,pwidth,pheight,pz+dpz,pdepth)) continue

				// ix, iy, and iz measure how far you are inside the turf in each direction.
				var/ix = 0
				var/iy = 0
				var/iz = 0

				// If you draw pictures showing a mob hitting a dense turf from the left
				// side and label px, dpx, pwidth, and t.px it's easy to see how you
				// compute ix. The same can be done for hitting a dense turf from the right.
				if(dpx > 0)
					ix = px + dpx + pwidth - t.px
				else if(dpx < 0)
					ix = (px + dpx) - (t.px + t.pwidth)

				// Same as the ix calculations except we swap y for x and height for width.
				if(dpy > 0)
					iy = py + dpy + pheight - t.py
				else if(dpy < 0)
					iy = (py + dpy) - (t.py + t.pheight)

				// we only care about iz if we're falling. there are no ceilings
				if(dpz > 0)
					iz = pz + dpz + pdepth - t.pz
					// iz = (pz + dpz) - (t.pz + t.pdepth)
				else if(dpz < 0)
					iz = (pz + dpz) - (t.pz + t.pdepth)
					// iz = dpz * 2000

				// tx, ty, and tz measure the fraction of the move that it takes
				// for you to hit the turf in each direction.
				var/tx = (dpx == 0) ? 1000 : ix / dpx
				var/ty = (dpy == 0) ? 1000 : iy / dpy
				var/tz = (dpz == 0) ? 1000 : iz / dpz

				// We use tx, ty, and tz to determine if you first hit the object in the x, y, or z
				// direction. We modify dpx, dpy, and/or dpz based on how you bumped the turf.
				if(tz == min(tx,ty,tz,999))
					dpz -= iz
					z_bump()

				if(pz + dpz < t.pz + t.pdepth)
					if(ty == min(tx,ty,tz,999))
						dpy -= iy
						y_bump()
					else if(tx == min(tx,ty,tz,999))
						dpx -= ix
						x_bump()

			// At this point we've clipped your move against all nearby tiles, so the
			// move (dpx,dpy,dpz) is a valid one at this point (both might be zero) so we
			// can perform the move.
			set_pos(px + dpx, py + dpy, pz + dpz)

			if(dpx == 0 && dpy == 0 && dpz == 0)
				return 0
			return 1

		// Set pos computes your mob's x/y given your px/py, sets your loc, and manages the camera.
		set_pos(nx,ny,nz,map_z = -1)
			if(map_z == -1) map_z = z

			var/moved = (nx != px || ny != py || nz != pz || map_z != z)

			px = nx
			py = ny
			pz = nz

			var/tx = round((nx + pwidth / 2) / world.icon_size)
			var/ty = round((ny + pheight / 2) / world.icon_size)

			if(moved)
				var/turf/t = locate(tx,ty,map_z)
				if(loc != t)
					loc = t
					Move(loc)

				pixel_x = px - x * world.icon_size + offset_x
				pixel_y = py - y * world.icon_size + offset_y
				pixel_z = pz

				if(ismob(src))
					src:set_camera()

			var/list/ground = get_ground()
			for(var/atom/a in ground)
				a.stepped_on(src)

			last_x = x
			last_y = y
			last_z = z

mob
	proc
		set_camera()
			if(client)
				if(client.perspective == EDGE_PERSPECTIVE)
					if(x > client.view && x <= world.maxx - client.view)
						client.pixel_x = pixel_x
					if(y > client.view && y <= world.maxy - client.view)
						client.pixel_y = pixel_y
				else
					client.pixel_x = pixel_x
					client.pixel_y = pixel_y
